<?php

namespace Drupal\commerce_winbank_redirect\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Endpoints for the routes defined.
 */
class CallbackController extends ControllerBase {

  /**
   * The entity_type.manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The controller constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity_type.manager service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger.factory service.
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager, LoggerChannelFactoryInterface $logger_factory) {
    $this->entityTypeManager = $entityTypeManager;
    $this->logger = $logger_factory->get('commerce_winbank_redirect');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('logger.factory')
    );
  }

  /**
   * Callback action.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   */
  public function callback(Request $request) {
    $action = $request->get("action");

    if ($action == NULL) {
      return $this->createRedirectForm($request);
    }

    return $this->processCallback($request);
  }

  /**
   * Creates the redirect form.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return array
   *   Returns an array with the HTML form.
   */
  protected function createRedirectForm(Request $request) {
    $form = $this->formBuilder()->getForm('Drupal\commerce_winbank_redirect\Form\StaticRedirectForm');
    $exclude_keys = [
      "op",
      "form_id",
      "form_token",
      "form_build_id",
      "PaymentUrl",
    ];
    foreach ($request->request->all() as $key => $value) {
      if (in_array($key, $exclude_keys, TRUE)) {
        continue;
      }
      $form[$key] = [
        '#name' => $key,
        '#value' => $value,
        '#type' => 'hidden',
      ];
    }

    $form['#action'] = $request->get('PaymentUrl');
    $form['#attributes']['class'][] = 'payment-redirect-form';
    $form['#attached']['library'][] = 'commerce_payment/offsite_redirect';
    return $form;
  }

  /**
   * Process the callback from winbank.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   Returns a redirect response.
   */
  public function processCallback(Request $request) {
    $results = $request->request->all();
    $this->logger->notice(json_encode($results));
    $hashkey = $request->get('HashKey');
    $order_id = $request->get('MerchantReference');

    $options = [];
    try {
      $parameters = $request->get('Parameters');
      $params = [];
      $values = explode(';', $parameters);
      foreach ($values as $kv) {
        [$k, $v] = explode(':', $kv);
        $params[$k] = $v;
      }
      $langcode = $params['lang'];
      if ($language = $this->languageManager()->getLanguage($langcode)) {
        $options['language'] = $language;
      }
    }
    catch (\Exception $e) {
    }

    $action = $request->get("action");
    if ($action === "cancel") {
      return new RedirectResponse(Url::fromRoute('commerce_payment.checkout.cancel', [
        'commerce_order' => $order_id,
        'step' => 'payment',
      ], $options)->toString());
    }

    $resultCode = $request->get('ResultCode');
    if ($resultCode !== "0") {
      $this->logResponse($request, "failure");
      $this->messenger()->addError($this->getDisplayMessage($request));
      return new RedirectResponse(Url::fromRoute('commerce_checkout.form', [
        'commerce_order' => $order_id,
        'step' => 'order_information',
      ], $options)->toString());
    }
    /** @var \Drupal\commerce_order\Entity\OrderInterface $order */
    $order = $this->entityTypeManager->getStorage('commerce_order')->load($order_id);
    $ticket_result = $order->getData("IssueNewTicketResult");
    $hash = $this->calculateHash($request, $order_id, $ticket_result);
    if (strtolower($hash) !== strtolower($hashkey)) {
      $this->createPayment($order, $request, $ticket_result, "Unvalidated");
      $this->logResponse($request, "failure");
      $this->messenger()->addError($this->t("Sorry we were not able to validate your payment."));
      return new RedirectResponse(Url::fromRoute('commerce_checkout.form', [
        'commerce_order' => $order_id,
        'step' => 'order_information',
      ], $options)->toString());
    }

    $this->createPayment($order, $request, $ticket_result);
    $this->logResponse($request, "success");
    return new RedirectResponse(Url::fromRoute('commerce_checkout.form', [
      'commerce_order' => $order_id,
      'step_id' => 'complete',
    ], $options)->toString());
  }

  /**
   * Log response data to database log and order object.
   */
  public function logResponse($request, $status = "success") {
    $order_id = $request->get('MerchantReference');
    $data = [
      "SupportReferenceID" => $request->get('SupportReferenceID'),
      "MerchantReference" => $order_id,
      "ResponseCode" => $request->get('ResponseCode'),
      "ResponseDescription" => $request->get('ResponseDescription'),
    ];

    if ($status === "success") {
      $data2 = [
        "StatusFlag" => $request->get('StatusFlag'),
        "ApprovalCode" => $request->get('ApprovalCode'),
        "PackageNo" => $request->get('PackageNo'),
        "AuthStatus" => $request->get('AuthStatus'),
      ];
      $data = array_merge($data, $data2);
    }

    // Log for immediate action.
    if ($status === "success") {
      $this->logger->info("Payment " . $status . ": " . '<pre><code>' . print_r($data, TRUE) . '</code></pre>');
    }
    else {
      if ($status === "failure") {
        $this->logger->info("Payment " . $status . ": " . '<pre><code>' . print_r($data, TRUE) . '</code></pre>');
      }
    }

    // Log in the order data field for future reference.
    /** @var \Drupal\commerce_order\Entity\OrderInterface $order */
    $order = $this->entityTypeManager->getStorage('commerce_order')->load($order_id);
    // phpcs:disable
    $IssueNewTicketResult = $order->getData("IssueNewTicketResult");
    $IssueNewTicketResult = array_merge($IssueNewTicketResult, $data);
    $order->setData("IssueNewTicketResult", $IssueNewTicketResult);
    // phpcs:enable
    $order->save();
  }

  /**
   * Creates the payment for the order.
   */
  public function createPayment($order, $request, $ticket_result, $state = "completed") {
    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = $this->entityTypeManager->getStorage('commerce_payment')->create([
      'state' => $state,
      'amount' => $order->getBalance(),
      'payment_gateway' => $ticket_result["payment_gateway"],
      'order_id' => $order->id(),
      'remote_id' => $request->get('TransactionId'),
      'remote_state' => $request->get('StatusFlag'),
    ]);
    if ($state === "completed") {
      $payment->setAuthorizedTime(\Drupal::time()->getRequestTime());
      $payment->setCompletedTime(\Drupal::time()->getRequestTime());
    }
    $payment->save();
  }

  /**
   * Calculates hash key by concatenation of values, then uses sha256 algorithm.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object containing the required data.
   * @param int $order_id
   *   The ID of the order.
   * @param array $ticket_result
   *   The values retrieved from the ticketing service.
   *
   * @return string
   *   Returns the hash key to be used for the validation of the response.
   */
  public function calculateHash(Request $request, $order_id, array $ticket_result) {
    $concatValues = sprintf(
      '%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s',
      $ticket_result['TranTicket'],
      $ticket_result['PosId'],
      $ticket_result['AcquirerId'],
      $order_id,
      $request->get('ApprovalCode'),
      $request->get('Parameters'),
      $request->get('ResponseCode'),
      $request->get('SupportReferenceID'),
      $request->get('AuthStatus'),
      $request->get('PackageNo'),
      $request->get('StatusFlag')
    );
    return hash_hmac('sha256', $concatValues, $ticket_result['TranTicket'], FALSE);
  }

  /**
   * Get message from request results.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object with the data.
   *
   * @return string
   *   Returns the message to display to the end user.
   */
  public function getDisplayMessage(Request $request) {
    $message = $this->t('Transaction not complete.');
    $resultCode = $request->get('ResultCode');
    $resultCodes = [
      '1048' => $this->t('Transaction not complete. The transaction has previously been processed.'),
      '500' => $this->t('Transaction not complete because of technical problem. Please try again later.'),
      '981' => $this->t('Transaction not complete. Card details not correct or bank does not support type of card.'),
      '1045' => $this->t('Transaction not complete. The transaction is probably still in process.'),
      '1072' => $this->t('Transaction not complete because of technical problem. Please try again later.'),
      '1' => $this->t('Transaction not complete because of technical problem. Please try again later.'),
    ];
    if (isset($resultCodes[$resultCode])) {
      $message = $resultCodes[$resultCode];
    }
    elseif ($resultCode === '0') {
      $responseCode = $request->get('ResponseCode');
      if ($responseCode == '00') {
        $message = $this->t('Your transaction was completed. Thank you.');
      }
      elseif ($responseCode == '12') {
        $message = $this->t('Your transaction was denied by the issuing bank. Can retry with a different card or contact support.');
      }
    }
    return $message;
  }

}
